@@ -38,9 +38,8 @@
 
 #include <gdata/gdata.h>
 
-#define GTKBUILDER_CONFIG_FILE EOG_PLUGINDIR"/postasa/postasa-config.xml"
-#define GTKBUILDER_UPLOAD_FILE EOG_PLUGINDIR"/postasa/postasa-uploads.xml"
-#define DEFAULT_THUMBNAIL EOG_PLUGINDIR"/postasa/default.png"
+#define GTKBUILDER_CONFIG_FILE EOG_POSTASA_DATA_DIR"/postasa-config.xml"
+#define GTKBUILDER_UPLOAD_FILE EOG_POSTASA_DATA_DIR"/postasa-uploads.xml"
 
 enum {
 	PROP_O,
@@ -67,6 +66,9 @@ struct _EogPostasaPluginPrivate
 	guint ui_id;
 
 
+#ifdef HAVE_LIBGDATA_0_9
+	GDataClientLoginAuthorizer *authorizer;
+#endif
 	GDataPicasaWebService *service;
 	GCancellable *login_cancellable;
 
@@ -307,7 +309,8 @@ uploads_add_entry (EogPostasaPlugin *plugin, EogImage *image, GCancellable *canc
 			g_error_free (error);
 		}
 	}
-	size = g_strdup_printf ("%luKB", eog_image_get_bytes (image) / 1024);
+	size = g_strdup_printf ("%" G_GOFFSET_FORMAT "KB",
+				eog_image_get_bytes (image) / 1024);
 	iter = g_slice_new0 (GtkTreeIter);
 
 	/* insert the data into the upload's list store */
@@ -405,7 +408,7 @@ tmp_picasaweb_upload_async (GSimpleAsyncResult *result, GObject *source_object,
 	GDataPicasaWebService *service = plugin->priv->service;
 	GDataPicasaWebFile *file_entry;
 	PicasaWebUploadFileAsyncData *data;
-#ifdef HAVE_LIBGDATA_80
+#ifdef HAVE_LIBGDATA_0_8
 	GDataUploadStream *upload_stream;
 	GFileInputStream *in_stream;
 	GFileInfo *file_info;
@@ -421,7 +424,7 @@ tmp_picasaweb_upload_async (GSimpleAsyncResult *result, GObject *source_object,
 	gdata_entry_set_title (GDATA_ENTRY (file_entry), filename);
 	g_free (filename);
 
-#ifdef HAVE_LIBGDATA_80
+#ifdef HAVE_LIBGDATA_0_8
 	file_info = g_file_query_info (data->imgfile,
 				      G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
 				      G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
@@ -521,7 +524,11 @@ picasaweb_upload_files (EogPostasaPlugin *plugin)
 	GSimpleAsyncResult *result;
 	PicasaWebUploadFileAsyncData *data;
 
+#ifdef HAVE_LIBGDATA_0_9
+	if (gdata_service_is_authorized (GDATA_SERVICE (plugin->priv->service)) == FALSE) {
+#else
 	if (gdata_service_is_authenticated (GDATA_SERVICE (plugin->priv->service)) == FALSE) {
+#endif
 		g_warning ("PicasaWeb could not be authenticated.  Aborting upload.");
 		return;
 	}
@@ -564,14 +571,25 @@ picasaweb_upload_files (EogPostasaPlugin *plugin)
  * "Close".  Regardless of the response, it re-enables the Login
  * button (which is desensitised during the login attempt).
  **/
+#ifdef HAVE_LIBGDATA_0_9
+static void
+picasaweb_login_async_cb (GDataClientLoginAuthorizer *authorizer, GAsyncResult *result, EogPostasaPlugin *plugin)
+#else
 static void
 picasaweb_login_async_cb (GDataPicasaWebService *service, GAsyncResult *result, EogPostasaPlugin *plugin)
+#endif
 {
 	GError *error = NULL;
 	gchar *message;
 	gboolean success = FALSE;
 
+#ifdef HAVE_LIBGDATA_0_9
+	success = gdata_client_login_authorizer_authenticate_finish (authorizer,
+								     result,
+								     &error);
+#else
 	success = gdata_service_authenticate_finish (GDATA_SERVICE (service), result, &error);
+#endif
 
 	gtk_widget_set_sensitive (GTK_WIDGET (plugin->priv->login_button), TRUE);
 	gtk_widget_set_sensitive (GTK_WIDGET (plugin->priv->username_entry), TRUE);
@@ -612,10 +630,19 @@ picasaweb_login_cb (GtkWidget *login_button, gpointer _plugin)
 	/* TODO: want to handle passwords more securely */
 	gtk_label_set_text (plugin->priv->login_message, _("Logging in..."));
 	g_cancellable_reset (plugin->priv->login_cancellable);
+
+#ifdef HAVE_LIBGDATA_0_9
+	gdata_client_login_authorizer_authenticate_async (
+					  plugin->priv->authorizer,
+					  gtk_entry_get_text (plugin->priv->username_entry),
+					  gtk_entry_get_text (plugin->priv->password_entry),
+					  plugin->priv->login_cancellable, (GAsyncReadyCallback)picasaweb_login_async_cb, plugin);
+#else
 	gdata_service_authenticate_async (GDATA_SERVICE (plugin->priv->service),
 					  gtk_entry_get_text (plugin->priv->username_entry),
 					  gtk_entry_get_text (plugin->priv->password_entry),
 					  plugin->priv->login_cancellable, (GAsyncReadyCallback)picasaweb_login_async_cb, plugin);
+#endif
 }
 
 /**
@@ -633,7 +660,13 @@ picasaweb_upload_cb (GtkAction	*action,
 	g_return_if_fail (EOG_IS_POSTASA_PLUGIN (plugin));
 
 	priv = plugin->priv;
-	if (gdata_service_is_authenticated (GDATA_SERVICE (priv->service)) == TRUE) {
+
+#ifdef HAVE_LIBGDATA_0_9
+	if (gdata_service_is_authorized (GDATA_SERVICE (priv->service)) == TRUE)
+#else
+	if (gdata_service_is_authenticated (GDATA_SERVICE (priv->service)) == TRUE)
+#endif
+	{
 		picasaweb_upload_files (plugin);
 	} else {
 		/* when the dialog closes, it checks if this is set to see if it should upload anything */
@@ -679,6 +712,9 @@ login_dialog_close (EogPostasaPlugin *plugin)
 static gboolean
 login_dialog_cancel_button_cb (GtkWidget *cancel_button, EogPostasaPlugin *plugin)
 {
+	/* Make sure we don't resume uploads */
+	plugin->priv->uploads_pending = FALSE;
+
 	return login_dialog_close (plugin);
 }
 
@@ -690,6 +726,9 @@ login_dialog_cancel_button_cb (GtkWidget *cancel_button, EogPostasaPlugin *plugi
 static gboolean
 login_dialog_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer *_plugin)
 {
+	/* Make sure we don't resume uploads */
+	EOG_POSTASA_PLUGIN (_plugin)->priv->uploads_pending = FALSE;
+
 	return login_dialog_close (EOG_POSTASA_PLUGIN (_plugin));
 }
 
@@ -729,9 +768,15 @@ login_get_dialog (EogPostasaPlugin *plugin)
 		g_signal_connect (G_OBJECT (plugin->priv->cancel_button), "clicked", G_CALLBACK (login_dialog_cancel_button_cb), plugin);
 		g_signal_connect (G_OBJECT (plugin->priv->login_dialog), "delete-event", G_CALLBACK (login_dialog_delete_event_cb), plugin);
 
+#ifdef HAVE_LIBGDATA_0_9
+		if (gdata_service_is_authorized (GDATA_SERVICE (plugin->priv->service))) {
+			gtk_entry_set_text (plugin->priv->username_entry, gdata_client_login_authorizer_get_username (plugin->priv->authorizer));
+			gtk_entry_set_text (plugin->priv->password_entry, gdata_client_login_authorizer_get_password (plugin->priv->authorizer));
+#else
 		if (gdata_service_is_authenticated (GDATA_SERVICE (plugin->priv->service))) {
 			gtk_entry_set_text (plugin->priv->username_entry, gdata_service_get_username (GDATA_SERVICE (plugin->priv->service)));
 			gtk_entry_set_text (plugin->priv->password_entry, gdata_service_get_password (GDATA_SERVICE (plugin->priv->service)));
+#endif
 		}
 	}
 
@@ -813,7 +858,13 @@ eog_postasa_plugin_init (EogPostasaPlugin *plugin)
 	eog_debug_message (DEBUG_PLUGINS, "EogPostasaPlugin initializing");
 
 	plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE (plugin, EOG_TYPE_POSTASA_PLUGIN, EogPostasaPluginPrivate);
+
+#ifdef HAVE_LIBGDATA_0_9
+	plugin->priv->authorizer = gdata_client_login_authorizer_new ("EogPostasa", GDATA_TYPE_PICASAWEB_SERVICE);
+	plugin->priv->service = gdata_picasaweb_service_new (GDATA_AUTHORIZER (plugin->priv->authorizer)); /* unref'd in eog_postasa_plugin_dispose() */
+#else
 	plugin->priv->service = gdata_picasaweb_service_new ("EogPostasa"); /* unref'd in eog_postasa_plugin_dispose() */
+#endif
 	plugin->priv->login_cancellable = g_cancellable_new (); /* unref'd in eog_postasa_plugin_dispose() */
 	plugin->priv->uploads_pending = FALSE;
 }
@@ -830,6 +881,13 @@ eog_postasa_plugin_dispose (GObject *_plugin)
 
 	eog_debug_message (DEBUG_PLUGINS, "EogPostasaPlugin disposing");
 
+#ifdef HAVE_LIBGDATA_0_9
+	if (plugin->priv->authorizer) {
+		g_object_unref (plugin->priv->authorizer);
+		plugin->priv->authorizer = NULL;
+	}
+#endif
+
 	if (plugin->priv->service) {
 		g_object_unref (plugin->priv->service);
 		plugin->priv->service = NULL;
